"""
CLI Package Main Entry Point
============================

Entry point for running the CLI as a module with `python -m genebot.cli`
"""

from .main import main

if __name__ == '__main__':
    main()